@@ -3,12 +3,15 @@ from __future__ import annotations
 
 import copy
 import datetime
+from datetime import date
+from functools import lru_cache
 from operator import itemgetter
 from os.path import basename
 from os.path import dirname
 from os.path import join
 from os.path import normpath
 from typing import Iterable
+from typing import TYPE_CHECKING
 
 from beancount.core import realization
 from beancount.core.account_types import AccountTypes
@@ -44,6 +47,7 @@ from fava.core.accounts import get_entry_accounts
 from fava.core.attributes import AttributesModule
 from fava.core.budgets import BudgetModule
 from fava.core.charts import ChartModule
+from fava.core.commodities import CommoditiesModule
 from fava.core.entries_by_type import group_entries_by_type
 from fava.core.extensions import ExtensionModule
 from fava.core.fava_options import FavaOptions
@@ -61,11 +65,16 @@ from fava.core.tree import Tree
 from fava.core.watcher import Watcher
 from fava.helpers import BeancountError
 from fava.helpers import FavaAPIException
-from fava.util import date
 from fava.util import pairwise
+from fava.util.date import Interval
+from fava.util.date import interval_ends
 from fava.util.typing import BeancountOptions
 
 
+if TYPE_CHECKING:  # pragma: no cover
+    from beancount.core.prices import PriceMap
+
+
 class Filters:
     """The possible entry filters."""
 
@@ -105,6 +114,7 @@ MODULES = [
     "attributes",
     "budgets",
     "charts",
+    "commodities",
     "extensions",
     "file",
     "format_decimal",
@@ -114,6 +124,112 @@ MODULES = [
 ]
 
 
+class FilteredLedger:
+    """Filtered Beancount ledger."""
+
+    __slots__ = [
+        "ledger",
+        "entries",
+        "filters",
+        "root_account",
+        "root_tree",
+        "_date_first",
+        "_date_last",
+    ]
+    _date_first: date | None
+    _date_last: date | None
+
+    def __init__(
+        self,
+        ledger: FavaLedger,
+        account: str | None = None,
+        filter: str | None = None,  # pylint: disable=redefined-builtin
+        time: str | None = None,
+    ):
+        self.ledger = ledger
+        self.filters = Filters(ledger.options, ledger.fava_options)
+        self.filters.set(account=account, filter=filter, time=time)
+        self.entries = self.filters.apply(ledger.all_entries)
+
+        self.root_account = realization.realize(
+            self.entries, ledger.account_types
+        )
+        self.root_tree = Tree(self.entries)
+
+        self._date_first, self._date_last = get_min_max_dates(
+            self.entries, (Transaction, Price)
+        )
+        if self._date_last:
+            self._date_last = self._date_last + datetime.timedelta(1)
+
+        if self.filters.time:
+            self._date_first = self.filters.time.begin_date
+            self._date_last = self.filters.time.end_date
+
+    @property
+    def end_date(self) -> date | None:
+        """The date to use for prices."""
+        if self.filters.time:
+            return self.filters.time.end_date
+        return None
+
+    @property
+    def root_tree_closed(self) -> Tree:
+        """A root tree for the balance sheet."""
+        tree = Tree(self.entries)
+        tree.cap(self.ledger.options, self.ledger.fava_options.unrealized)
+        return tree
+
+    def interval_ends(self, interval: Interval) -> Iterable[date]:
+        """Generator yielding dates corresponding to interval boundaries."""
+        if not self._date_first or not self._date_last:
+            return []
+        return interval_ends(self._date_first, self._date_last, interval)
+
+    @property
+    def documents(self) -> list[Document]:
+        """All currently filtered documents."""
+        return [e for e in self.entries if isinstance(e, Document)]
+
+    def events(self) -> list[Event]:
+        """All currently filtered events."""
+        return [e for e in self.entries if isinstance(e, Event)]
+
+    def prices(self, base: str, quote: str) -> list[tuple[date, Decimal]]:
+        """List all prices."""
+        all_prices = get_all_prices(self.ledger.price_map, (base, quote))
+
+        if (
+            self.filters.time
+            and self.filters.time.begin_date is not None
+            and self.filters.time.end_date is not None
+        ):
+            return [
+                (date_, price)
+                for date_, price in all_prices
+                if self.filters.time.begin_date
+                <= date_
+                < self.filters.time.end_date
+            ]
+        return all_prices
+
+    def account_is_closed(self, account_name: str) -> bool:
+        """Check if the account is closed.
+
+        Args:
+            account_name: An account name.
+
+        Returns:
+            True if the account is closed before the end date of the current
+            time filter.
+        """
+        if self.filters.time and self._date_last is not None:
+            return (
+                self.ledger.accounts[account_name].close_date < self._date_last
+            )
+        return self.ledger.accounts[account_name].close_date != date.max
+
+
 # pylint: disable=too-many-instance-attributes,too-many-public-methods
 class FavaLedger:
     """Create an interface for a Beancount ledger.
@@ -129,30 +245,23 @@ class FavaLedger:
         "all_entries_by_type",
         "all_root_account",
         "beancount_file_path",
-        "commodity_names",
-        "_date_first",
-        "_date_last",
-        "entries",
         "errors",
         "fava_options",
-        "filters",
         "_is_encrypted",
         "options",
         "price_map",
-        "root_account",
-        "root_tree",
         "_watcher",
     ] + MODULES
 
     #: List of all (unfiltered) entries.
     all_entries: Entries
 
-    #: The entries filtered according to the chosen filters.
-    entries: Entries
-
     #: A NamedTuple containing the names of the five base accounts.
     account_types: AccountTypes
 
+    #: The price map.
+    price_map: PriceMap
+
     def __init__(self, path: str) -> None:
         #: The path to the main Beancount file.
         self.beancount_file_path = path
@@ -167,6 +276,9 @@ class FavaLedger:
         #: A :class:`.ChartModule` instance.
         self.charts = ChartModule(self)
 
+        #: A :class:`.CommoditiesModule` instance.
+        self.commodities = CommoditiesModule(self)
+
         #: A :class:`.ExtensionModule` instance.
         self.extensions = ExtensionModule(self)
 
@@ -202,15 +314,9 @@ class FavaLedger:
         #: A dict containing information about the accounts.
         self.accounts = AccountDict()
 
-        #: A dict with commodity names (from the 'name' metadata)
-        self.commodity_names: dict[str, str] = {}
-
         #: A dict with all of Fava's option values.
         self.fava_options: FavaOptions = FavaOptions()
 
-        self._date_first: datetime.date | None = None
-        self._date_last: datetime.date | None = None
-
         self.load_file()
 
     def load_file(self) -> None:
@@ -226,6 +332,8 @@ class FavaLedger:
                 self.beancount_file_path
             )
 
+        self.get_filtered.cache_clear()
+
         self.account_types = get_account_types(self.options)
         self.price_map = build_price_map(self.all_entries)
         self.all_root_account = realization.realize(
@@ -240,12 +348,6 @@ class FavaLedger:
         for close in self.all_entries_by_type.Close:
             self.accounts.setdefault(close.account).close_date = close.date
 
-        self.commodity_names = {}
-        for commodity in self.all_entries_by_type.Commodity:
-            name = commodity.meta.get("name")
-            if name:
-                self.commodity_names[commodity.currency] = name
-
         self.fava_options, errors = parse_options(
             self.all_entries_by_type.Custom
         )
@@ -257,47 +359,17 @@ class FavaLedger:
         for mod in MODULES:
             getattr(self, mod).load_file()
 
-        self.filters = Filters(self.options, self.fava_options)
-
-        self.filter(True)
-
-    # pylint: disable=attribute-defined-outside-init
-    def filter(
+    @lru_cache(maxsize=16)
+    def get_filtered(
         self,
-        force: bool = False,
         account: str | None = None,
         filter: str | None = None,  # pylint: disable=redefined-builtin
         time: str | None = None,
-    ) -> None:
-        """Set and apply (if necessary) filters."""
-        changed = self.filters.set(account=account, filter=filter, time=time)
-
-        if not (changed or force):
-            return
-
-        self.entries = self.filters.apply(self.all_entries)
-
-        self.root_account = realization.realize(
-            self.entries, self.account_types
+    ) -> FilteredLedger:
+        """Filter the ledger."""
+        return FilteredLedger(
+            ledger=self, account=account, filter=filter, time=time
         )
-        self.root_tree = Tree(self.entries)
-
-        self._date_first, self._date_last = get_min_max_dates(
-            self.entries, (Transaction, Price)
-        )
-        if self._date_last:
-            self._date_last = self._date_last + datetime.timedelta(1)
-
-        if self.filters.time:
-            self._date_first = self.filters.time.begin_date
-            self._date_last = self.filters.time.end_date
-
-    @property
-    def end_date(self) -> datetime.date | None:
-        """The date to use for prices."""
-        if self.filters.time:
-            return self.filters.time.end_date
-        return None
 
     def join_path(self, *args: str) -> str:
         """Path relative to the directory of the ledger."""
@@ -337,14 +409,6 @@ class FavaLedger:
             self.load_file()
         return changed
 
-    def interval_ends(
-        self, interval: date.Interval
-    ) -> Iterable[datetime.date]:
-        """Generator yielding dates corresponding to interval boundaries."""
-        if not self._date_first or not self._date_last:
-            return []
-        return date.interval_ends(self._date_first, self._date_last, interval)
-
     def get_account_sign(self, account_name: str) -> int:
         """Get account sign.
 
@@ -357,25 +421,17 @@ class FavaLedger:
         """
         return get_account_sign(account_name, self.account_types)
 
-    @property
-    def root_tree_closed(self) -> Tree:
-        """A root tree for the balance sheet."""
-        tree = Tree(self.entries)
-        tree.cap(self.options, self.fava_options.unrealized)
-        return tree
-
     def interval_balances(
         self,
-        interval: date.Interval,
+        filtered: FilteredLedger,
+        interval: Interval,
         account_name: str,
         accumulate: bool = False,
-    ) -> tuple[
-        list[realization.RealAccount],
-        list[tuple[datetime.date, datetime.date]],
-    ]:
+    ) -> tuple[list[realization.RealAccount], list[tuple[date, date]]]:
         """Balances by interval.
 
         Arguments:
+            filtered: The currently filtered ledger.
             interval: An interval.
             account_name: An account name.
             accumulate: A boolean, ``True`` if the balances for an interval
@@ -391,15 +447,15 @@ class FavaLedger:
         ]
 
         interval_tuples = list(
-            reversed(list(pairwise(self.interval_ends(interval))))
+            reversed(list(pairwise(filtered.interval_ends(interval))))
         )
 
         interval_balances = [
             realization.realize(
                 list(
                     iter_entry_dates(
-                        self.entries,
-                        datetime.date.min if accumulate else begin_date,
+                        filtered.entries,
+                        date.min if accumulate else begin_date,
                         end_date,
                     )
                 ),
@@ -411,11 +467,15 @@ class FavaLedger:
         return interval_balances, interval_tuples
 
     def account_journal(
-        self, account_name: str, with_journal_children: bool = False
+        self,
+        filtered: FilteredLedger,
+        account_name: str,
+        with_journal_children: bool = False,
     ) -> list[tuple[Directive, list[Posting], Inventory, Inventory]]:
         """Journal for an account.
 
         Args:
+            filtered: The currently filtered ledger.
             account_name: An account name.
             with_journal_children: Whether to include postings of subaccounts
                 of the given account.
@@ -425,7 +485,7 @@ class FavaLedger:
             change and balance have already been reduced to units.
         """
         real_account = realization.get_or_create(
-            self.root_account, account_name
+            filtered.root_account, account_name
         )
 
         if with_journal_children:
@@ -443,19 +503,6 @@ class FavaLedger:
             ) in realization.iterate_with_balance(postings)
         ]
 
-    @property
-    def documents(self) -> list[Document]:
-        """All currently filtered documents."""
-        return [e for e in self.entries if isinstance(e, Document)]
-
-    def events(self, event_type: str | None = None) -> list[Event]:
-        """List events (possibly filtered by type)."""
-        events = [e for e in self.entries if isinstance(e, Event)]
-        if event_type:
-            return [event for event in events if event.type == event_type]
-
-        return events
-
     def get_entry(self, entry_hash: str) -> Directive:
         """Find an entry.
 
@@ -531,26 +578,6 @@ class FavaLedger:
                 bw_pairs.append((currency_b, currency_a))
         return sorted(fw_pairs + bw_pairs)
 
-    def prices(
-        self, base: str, quote: str
-    ) -> list[tuple[datetime.date, Decimal]]:
-        """List all prices."""
-        all_prices = get_all_prices(self.price_map, (base, quote))
-
-        if (
-            self.filters.time
-            and self.filters.time.begin_date is not None
-            and self.filters.time.end_date is not None
-        ):
-            return [
-                (date, price)
-                for date, price in all_prices
-                if self.filters.time.begin_date
-                <= date
-                < self.filters.time.end_date
-            ]
-        return all_prices
-
     def last_entry(self, account_name: str) -> Directive | None:
         """Get last entry of an account.
 
@@ -617,20 +644,6 @@ class FavaLedger:
                 return "yellow"
         return None
 
-    def account_is_closed(self, account_name: str) -> bool:
-        """Check if the account is closed.
-
-        Args:
-            account_name: An account name.
-
-        Returns:
-            True if the account is closed before the end date of the current
-            time filter.
-        """
-        if self.filters.time and self._date_last is not None:
-            return self.accounts[account_name].close_date < self._date_last
-        return self.accounts[account_name].close_date != datetime.date.max
-
     @staticmethod
     def group_entries_by_type(entries: Entries) -> list[tuple[str, Entries]]:
         """Group the given entries by type.
